Package: lineagespot
Title: Detection of SARS-CoV-2 lineages in wastewater samples using
        next-generation sequencing
Version: 1.14.0
Date: 2021-03-24
Authors@R: c(
    person(given = "Nikolaos",
            family = "Pechlivanis",
            role = c("aut", "cre"),
            email = "inab.bioinformatics@lists.certh.gr",
            comment = c(ORCID = "0000-0003-2502-612X")),
    person("Maria", "Tsagiopoulou", email = "-", role = "aut"),
    person("Maria Christina", "Maniou", email = "-", role = "aut"),
    person("Anastasis", "Togkousidis", email = "-", role = "aut"),
    person("Evangelia", "Mouchtaropoulou", email = "-", role = "aut"),
    person("Taxiarchis", "Chassalevris", email = "-", role = "aut"),
    person("Serafeim", "Chaintoutis", email = "-", role = "aut"),
    person("Chrysostomos", "Dovas", email = "-", role = "aut"),
    person("Maria", "Petala", email = "-", role = "aut"),
    person("Margaritis", "Kostoglou", email = "-", role = "aut"),
    person("Thodoris", "Karapantsios", email = "-", role = "aut"),
    person("Stamatia", "Laidou", email = "-", role = "aut"),
    person("Elisavet", "Vlachonikola", email = "-", role = "aut"),
    person("Aspasia", "Orfanou", email = "-", role = "aut"),
    person("Styliani-Christina", "Fragkouli", email = "-", role = "aut"),
    person("Sofoklis", "Keisaris", email = "-", role = "aut"),
    person("Anastasia", "Chatzidimitriou", email = "-", role = "aut"),
    person("Agis", "Papadopoulos", email = "-", role = "aut"),
    person("Nikolaos", "Papaioannou", email = "-", role = "aut"),
    person("Anagnostis", "Argiriou", email = "-", role = "aut"),
    person("Fotis E.", "Psomopoulos", email = "-", role = "aut")
    )
Description: Lineagespot is a framework written in R, and aims to identify 
    SARS-CoV-2 related mutations based on a single (or a list) of variant(s) 
    file(s) (i.e., variant calling format). The method can facilitate the 
    detection of SARS-CoV-2 lineages in wastewater samples using next 
    generation sequencing, and attempts to infer the potential distribution 
    of the SARS-CoV-2 lineages. 
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.1.2
biocViews: VariantDetection, VariantAnnotation, Sequencing
Imports: VariantAnnotation, MatrixGenerics, SummarizedExperiment,
        data.table, stringr, httr, utils
Suggests: BiocStyle, RefManageR, rmarkdown, knitr, testthat (>= 3.0.0)
URL: https://github.com/BiodataAnalysisGroup/lineagespot
BugReports: https://github.com/BiodataAnalysisGroup/lineagespot/issues
BiocType: Software
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/lineagespot
git_branch: RELEASE_3_22
git_last_commit: c41b331
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:44:51 UTC; biocbuild
Author: Nikolaos Pechlivanis [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2502-612X>),
  Maria Tsagiopoulou [aut],
  Maria Christina Maniou [aut],
  Anastasis Togkousidis [aut],
  Evangelia Mouchtaropoulou [aut],
  Taxiarchis Chassalevris [aut],
  Serafeim Chaintoutis [aut],
  Chrysostomos Dovas [aut],
  Maria Petala [aut],
  Margaritis Kostoglou [aut],
  Thodoris Karapantsios [aut],
  Stamatia Laidou [aut],
  Elisavet Vlachonikola [aut],
  Aspasia Orfanou [aut],
  Styliani-Christina Fragkouli [aut],
  Sofoklis Keisaris [aut],
  Anastasia Chatzidimitriou [aut],
  Agis Papadopoulos [aut],
  Nikolaos Papaioannou [aut],
  Anagnostis Argiriou [aut],
  Fotis E. Psomopoulos [aut]
Maintainer: Nikolaos Pechlivanis <inab.bioinformatics@lists.certh.gr>

# Generated by roxygen2: do not edit by hand

export(get_lineage_report)
export(is_gff3)
export(lineagespot)
export(lineagespot_hits)
export(list_input)
export(list_vcf)
export(merge_vcf)
export(uniq_variants)
import(data.table)
importFrom(MatrixGenerics,rowRanges)
importFrom(SummarizedExperiment,assays)
importFrom(VariantAnnotation,fixed)
importFrom(VariantAnnotation,info)
importFrom(VariantAnnotation,readVcf)
importFrom(httr,GET)
importFrom(httr,content)
importFrom(stringr,str_detect)
importFrom(stringr,str_length)
importFrom(stringr,str_remove)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_squish)
importFrom(stringr,str_to_upper)
importFrom(utils,globalVariables)

# lineagespot


Lineagespot is a framework written in [R](https://www.r-project.org/), and aims to identify SARS-CoV-2 related mutations based on a single (or a list) of variant(s) file(s) (i.e., [variant calling format](https://gatk.broadinstitute.org/hc/en-us/articles/360035531692-VCF-Variant-Call-Format)). 


## Installation

```r
# install.packages("devtools")
devtools::install_github("BiodataAnalysisGroup/lineagespot")
```

## Raw data analysis

The processing steps of the raw fastq files can be found [here](inst/scripts/raw-data-analysis.md).

## Citation

If you use the tool, please cite the following work:

Pechlivanis, N., Tsagiopoulou, M., Maniou, M.C. et al. _Detecting SARS-CoV-2 lineages and mutational load in municipal wastewater and a use-case in the metropolitan area of Thessaloniki, Greece._ Sci Rep 12, 2659 (2022). https://doi.org/10.1038/s41598-022-06625-6

# `lineagespot` example datasets

## Raw fastq analysis

Example fastq files are provided through [zenodo](https://zenodo.org/badge/DOI/10.5281/zenodo.4564182.svg). 
For the pre processing steps of them, the bioinformatics analysis pipeline is provided [here](raw-data-analysis.md). 

The bioinformatics pipeline used for the analysis is based on the CDC ARTIC on Illumina Bioinformatic Workflow: https://github.com/CDCgov/SARS-CoV-2_Sequencing/tree/master/protocols/BFX-UT_ARTIC_Illumina

## NC_045512.2 annotation file

The provided gene annotation file (`GFF3`) has been downloaded from [NCBI SARS-CoV-2 resources](https://www.ncbi.nlm.nih.gov/sars-cov-2/). 

## Lineage reports references

The provided tab-delimited txt files containing lineage variants have been manually created based on [outbreak.info reports](https://outbreak.info/situation-reports).

#'
#'
#' uniq_variants
#'
#' @description
#' Lineage report for variants overlapping
#'
#' @param hits_table
#' A tab-delimited table containing the identified overlaps/hits between the 
#' input files and the lineages' reports. This input is generated by the 
#' `lineagespot_hits` function.
#'
#' @param AF_threshold
#' A parameter indicating the AF threshold that is going to applied in order to
#' identify the presence or not of a variant. This is used to compute the number
#' of variants in a sample and eventually the proportion of a lineage.
#'
#' @return A data table with metrics assessing the
#' abundance of every lineage in each samples
#'
#'
#' @export
#'
#' @examples
#'
#' variants_table <- merge_vcf(
#'     vcf_folder = system.file("extdata", "vcf-files",
#'         package = "lineagespot"
#'     ),
#'     gff3_path = system.file("extdata",
#'         "NC_045512.2_annot.gff3",
#'         package = "lineagespot"
#'     )
#' )
#' 
#' lineage_hits_table <- lineagespot_hits(
#'     vcf_table = variants_table,
#'     ref_folder = system.file("extdata", "ref",
#'         package = "lineagespot")
#' )
#' 
#' report <- uniq_variants(hits_table = lineage_hits_table)
#' head(report)
#' 


uniq_variants <- function(
    hits_table = NULL,
    AF_threshold = 0.8
    ) {
    if (is.null(hits_table)) {
        stop("Please provide a tab-delimited table")
    }

    # lineage overview -------------------------------------------------------

    lineage_stats <- unique(
        hits_table[, c("Gene_Name", "AA_alt", "lineage"), with = FALSE]
    )

    lineage_stats <- lineage_stats[, .N, by = lineage]

    count_lineages <- hits_table[which(hits_table$AF >= AF_threshold), ]

    count_lineages <- unique(
        count_lineages[, c(
            "Gene_Name", "AA_alt", "sample", "lineage"
        ), with = FALSE]
    )

    count_lineages <- count_lineages[, .N, by = .(lineage, sample)]

    # mean AF of all variants ------------------------------------------------

    meanAF <- hits_table[, .(meanAF = mean(AF)), by = .(lineage, sample)]


    # Find unique variants per lineage ---------------------------------------

    count_uniq <- hits_table[, c(
        "Gene_Name", "AA_alt", "lineage"
    ), with = FALSE]
    
    count_uniq <- unique(count_uniq)
    count_uniq <- count_uniq[order(count_uniq$Gene_Name, count_uniq$AA_alt), ]
    count_uniq <- count_uniq[, .N, by = .(Gene_Name, AA_alt)]


    count_uniq <- count_uniq[which(count_uniq$N == 1), ]

    hits_table_uniq <- hits_table[which(
        hits_table$Gene_Name %in% count_uniq$Gene_Name &
            hits_table$AA_alt %in% count_uniq$AA_alt
    ), ]

    # mean AF of unique variants ---------------------------------------------

    meanAF_uniq <- hits_table_uniq[, .(
        meanAF_uniq = mean(AF)
    ), by = .(lineage, sample)]

    # filter out zero-AF variants --------------------------------------------

    hits_table_uniq <- hits_table_uniq[which(hits_table_uniq$AF != 0), ]

    # min AF of non zeror unique variants

    minAF_uniq <- hits_table_uniq[, .(
        minAF_uniq_nonzero = min(AF)
    ), by = .(lineage, sample)]

    # create output table ----------------------------------------------------

    overall <- rbindlist(
        list(meanAF, meanAF_uniq, minAF_uniq, count_lineages),
        use.names <- TRUE,
        fill <- TRUE
    )

    overall <- overall[order(overall$lineage, overall$sample), ]

    overall <- overall[, lapply(
        .SD, sum,
        na.rm = TRUE
    ), by = .(lineage, sample)]

    overall[which(overall$minAF_uniq_nonzero == 0), ]$minAF_uniq_nonzero <- NA

    who <- match(overall$lineage, lineage_stats$lineage)

    overall$`lineage N. rules` <- lineage_stats[who, ]$N

    overall$`lineage prop.` <- overall$N / overall$`lineage N. rules`

    return(overall)
}

library(lineagespot)
test_that(' Lineagespot hits and uniq variants functions run properly',{
  
    lineage_hits_table <- lineagespot_hits(
        vcf_table = merge_vcf(
            vcf_folder = system.file("extdata", "vcf-files",
                                     package = "lineagespot"
            ),
            gff3_path = system.file("extdata",
                                    "NC_045512.2_annot.gff3",
                                    package = "lineagespot"
            )
        ),
        ref_folder = system.file("extdata", "ref",
            package = "lineagespot")
    )
    
    expect_type( uniq_variants(hits_table = lineage_hits_table),'list')
    expect_type(lineage_hits_table,'list')
})

#'
#' lineagespot_hits
#'
#' @description
#' Find overlapping variants with SARS-CoV-2 reference lineages
#' coming from outbreak.info reports
#'
#' @param vcf_table
#' A tab-delimited table containing all variants for all samples. This input 
#' is generated by the `merge_vcf` function. 
#'
#' @param ref_folder
#' A path to lineages' reports
#'
#' @param voc
#' A character vector containing the names of the lineages of interest
#'
#'
#' @return
#' A data table containing all identified SARS-CoV-2 variants
#' based on the provided reference files
#'
#' @export
#'
#' @examples
#'
#' variants_table <- merge_vcf(
#'     vcf_folder = system.file("extdata",
#'         "vcf-files",
#'         package = "lineagespot"
#'     ),
#'     gff3_path = system.file("extdata",
#'         "NC_045512.2_annot.gff3",
#'         package = "lineagespot"
#'     )
#' )
#'
#' # retrieve lineage reports using outbreak.info's API
#'

#' # use user-specified references
#' lineage_hits_table <- lineagespot_hits(
#'     vcf_table = variants_table,
#'     ref_folder = system.file("extdata", "ref",
#'         package = "lineagespot"
#'     )
#' )
lineagespot_hits <- function(
    vcf_table = NULL,
    ref_folder = NULL,
    voc = c("B.1.617.2", "B.1.1.7", "B.1.351", "P.1")
    ) {
    if (is.null(vcf_table)) {
        stop("Please provide a tab-delimited table containing variants.")
    }
    if (is.null(ref_folder)) {
        stop("Please provide a path to lineages' reports.")
    }

    # clean AA variants ------------------------------------------------------

    aa_variants <- str_split(vcf_table$AA_alt, "[0-9]|_")
    codon_num <- str_split(vcf_table$AA_alt, "[A-Z]|[a-z]|\\_|\\*|\\?")

    aa_variants <- lapply(aa_variants, function(x) {
        return(x[x != ""])
    })

    codon_num <- lapply(codon_num, function(x) {
        return(x[x != ""])
    })


    # merge overall tables ----------------------------------------------------

    aa_split_list <- lapply(seq_len(nrow(vcf_table)), function(i) {
        out <- data.table(
            ref_aa = NA,
            alt_aa = NA,
            info = NA,
            codon_num = NA,
            codon_start = NA,
            codon_end = NA
        )


        if (length(aa_variants[[i]]) == 2) {
            out$ref_aa <- aa_variants[[i]][1]
            out$alt_aa <- aa_variants[[i]][2]

            out$codon_num <- codon_num[[i]][1]
        } else if (length(aa_variants[[i]] == 3)) {
            out$ref_aa <- aa_variants[[i]][1]
            out$alt_aa <- aa_variants[[i]][2]

            out$type == aa_variants[[i]][3]

            out$codon_start <- codon_num[[i]][1]
            out$codon_end <- codon_num[[i]][2]
        } else {


        }

        return(out)
    })

    aa_split_list <- rbindlist(aa_split_list)

    aa_split_list <- aa_split_list[, c(
        "ref_aa",
        "alt_aa",
        "info",
        "codon_start",
        "codon_end"
    ), with = FALSE]

    vcf_table <- cbind(vcf_table, aa_split_list)

    vcf_table$change_length_nt <- abs(
        str_length(vcf_table$ALT) - str_length(vcf_table$REF)
    )

    vcf_table$codon_num <- as.numeric(vcf_table$codon_num)
    vcf_table$codon_start <- as.numeric(vcf_table$codon_start)
    vcf_table$codon_end <- as.numeric(vcf_table$codon_end)


    # read reference ---------------------------------------------------------

    if (!is.null(ref_folder)) {
        fls_ref <- list.files(
            ref_folder,
            pattern = "txt",
            full.names = TRUE
        )

        reference_list <- list()

        for (i in fls_ref) {
            ref <- fread(i)

            ref <- ref[order(ref$gene), ]

            ref$barcode <- paste0(ref$gene, ":", ref$`amino acid`)

            ref$type <- "snp"

            ref[which(str_detect(ref$`amino acid`, "del")), ]$type <- "deletion"

            reference_list[[i]] <- ref
        }
    } else {
        reference_list <- get_lineage_report(lineages = voc)
    }

    VoC_hits_list <- list()

    for (ref_index in names(reference_list)) {
        ref <- reference_list[[ref_index]]

        aa_variants <- str_split(ref$`amino acid`, "[0-9]|_|\\/")
        codon_num <- str_split(ref$`amino acid`, "[A-Z]|[a-z]|\\_|\\*|\\?|\\/")

        aa_variants <- lapply(aa_variants, function(x) {
            x <- x[x != ""]
            x <- str_replace_all(x, "\\*", "\\\\*")

            return(x)
        })

        codon_num <- lapply(codon_num, function(x) {
            return(as.numeric(x[x != ""]))
        })

        # VoC hits ------------------------------------------------------------

        voc_data <- list()

        for (i in seq_len(nrow(ref))) {
            if (ref[i, ]$type == "snp") {
                temp <- vcf_table[which(vcf_table$Gene_Name == ref[i, ]$gene), ]

                temp <- temp[which(
                    str_detect(temp$ref_aa, aa_variants[[i]][1])
                ), ]

                temp <- temp[which(
                    str_detect(temp$alt_aa, aa_variants[[i]][2])
                ), ]

                who <- which(
                    temp$codon_num == codon_num[[i]] |
                        temp$codon_num == codon_num[[i]] + 1 |
                        temp$codon_num == codon_num[[i]] - 1
                )

                temp <- temp[who, ]
            } else {
                temp <- vcf_table[which(vcf_table$Gene_Name == ref[i, ]$gene), ]

                temp <- temp[which(str_detect(temp$AA_alt, "del")), ]

                who <- which(
                    temp$codon_start == codon_num[[i]][1] |
                        temp$codon_start == codon_num[[i]][2]
                )

                temp <- temp[who, ]

                if (nrow(temp) != 0) {
                    temp$AA_alt <- ref[i, ]$`amino acid`
                }
            }


            voc_data[[ref[i, ]$barcode]] <- temp
        }

        not_overlapping_variants <- names(which(lapply(voc_data, nrow) == 0))


        voc_data <- rbindlist(voc_data)

        voc_data <- unique(voc_data)



        # collapse table ------------------------------------------------------

        voc_data <- voc_data[, c(
            "POS",
            "DP",
            "AD_alt",
            "Gene_Name",
            "AA_alt",
            "sample"
        ), with = FALSE]

        voc_data <- voc_data[, .(
            DP = unique(DP),
            AD_alt = sum(AD_alt)
        ), by = .(POS, Gene_Name, AA_alt, sample)]

        voc_data <- voc_data[, .(
            DP = sum(DP),
            AD_alt = sum(AD_alt)
        ), by = .(Gene_Name, AA_alt, sample)]


        voc_data$AF <- voc_data$AD_alt / voc_data$DP

        # add non overlapping rules -------------------------------------------


        not_overlapping_variants <- str_split(
            not_overlapping_variants, "\\:",
            simplify = TRUE
        )

        voc_data <- rbind(
            voc_data,
            data.table(
                Gene_Name = rep(
                    not_overlapping_variants[, 1],
                    length(unique(voc_data$sample))
                ),
                AA_alt = rep(
                    not_overlapping_variants[, 2],
                    length(unique(voc_data$sample))
                ),
                sample = unique(voc_data$sample),
                DP = 0,
                AD_alt = 0,
                AF = 0
            )
        )

        VoC_hits_list[[ref_index]] <- voc_data
    }

    for (i in names(VoC_hits_list)) {
        strain <- str_split(i, "\\/", simplify = TRUE)
        strain <- strain[, ncol(strain)]
        strain <- str_remove(strain, "\\.txt")

        VoC_hits_list[[i]]$lineage <- strain
    }

    voc_data <- rbindlist(VoC_hits_list)

    
    return(voc_data)
}

#'
#' merge_vcf
#'
#' @description
#' Merge Variant Calling Format (VCF) files into a single tab-delimited table
#'
#' @param vcf_fls
#' A list of paths to VCF files
#'
#' @param vcf_folder
#' A path to a folder containing all VCF file that
#' will be integrated into a single table
#'
#' @param gff3_path
#' Path to GFF3 file
#'
#'
#' @return
#' A data table contaiing all variants from each sample of the input VCF files
#'
#' @export
#'
#' @examples
#'
#' merge_vcf(
#'     vcf_folder = system.file("extdata",
#'         "vcf-files",
#'         package = "lineagespot"
#'     ),
#'     gff3_path = system.file("extdata",
#'         "NC_045512.2_annot.gff3",
#'         package = "lineagespot"
#'     )
#' )
merge_vcf <- function(
    vcf_fls = NULL,
    vcf_folder = NULL,
    gff3_path = NULL
) {
    
 
    vcf_fls <- list_input(vcf_fls,vcf_folder,gff3_path)

    # Read input VCF files ----------------------------------------------------
    vcf_list <- list()

    for (i in vcf_fls) {

        # vcf_list[[i]] = vcfR::read.vcfR(i, verbose = FALSE)

        vcf_list[[i]] <- readVcf(i)
    }

    rm(i)


    # Convert VCF to table ----------------------------------------------------

    vcf_list <- lapply(vcf_list, vcf_to_table)

    # break multiple rules ----------------------------------------------------

    vcf_list <- lapply(vcf_list, break_multiple_variants)

    # add variant parameters --------------------------------------------------

    vcf_list <- lapply(vcf_list, compute_AF)

    # add variant parameters --------------------------------------------------

    vcf_list <- lapply(vcf_list, change_AA_abbreviations)

    # add variant parameters --------------------------------------------------

    genes <- read_gene_coordinates(gff_path = gff3_path)

    vcf_list <- lapply(vcf_list, correct_Orf1ab_gene, genes)

    # add sample name ---------------------------------------------------------

    for (i in names(vcf_list)) {
        sample_name <- unlist(str_split(i, "\\/"))
        sample_name <- sample_name[length(sample_name)]

        sample_name <- str_split(sample_name, "\\.vcf", simplify = TRUE)[, 1]

        vcf_list[[i]]$sample <- sample_name
    }

    # Combine all vcfs into one table -----------------------------------------

    vcf_list <- rbindlist(vcf_list)

    return(vcf_list)
}

vcf_to_table <- function(x) {

    # out <- cbind(x@fix[, seq_len(7)],
    # vcfR::extract_gt_tidy(x, verbose = FALSE),
    # vcfR::extract_info_tidy(x))

    out <- cbind(
        as.data.frame(rowRanges(x)),
        as.data.frame(fixed(x)),
        as.data.frame(info(x))
    )

    out$ALT <- unlist(lapply(out$ALT, function(x) {
        return(paste(as.character(x), collapse = ","))
    }))

    out$ANN <- unlist(lapply(out$ANN, function(x) {
        return(x[1])
    }))

    out$ID <- row.names(out)

    out$AD <- unlist(lapply(assays(x)[["AD"]], function(x) {
        return(paste(x, collapse = ","))
    }))

    out <- setDT(out)

    out <- out[which(out$ALT != "<*>"), ]

    ANN_matrix <- str_split(out$ANN, "\\|", simplify = TRUE)[, c(4, 10, 11)]

    # ANN_matrix <- paste0(ANN_matrix[,1], "|", ANN_matrix[,2])

    values_wewant <- c(
        "seqnames", # "CHROM",
        "start", # "POS",
        "ID",
        "REF",
        "ALT",
        "DP", # "gt_DP",
        "AD"
    ) # "gt_AD")

    result <- intersect(values_wewant, colnames(out))

    out <- out[, result, with = FALSE]

    if (identical((intersect(result, "gt_DP")), "gt_DP")) {
        names(out)[names(out) == "gt_DP"] <- "DP"
    }

    if (identical((intersect(result, "gt_AD")), "gt_AD")) {
        names(out)[names(out) == "gt_AD"] <- "AD"
    }

    colnames(out) <- c("CHROM", "POS", "ID", "REF", "ALT", "DP", "AD")

    out$CHROM <- str_squish(out$CHROM)
    out$Gene_Name <- ANN_matrix[, 1]
    out$Nt_alt <- ANN_matrix[, 2]
    out$AA_alt <- ANN_matrix[, 3]

    return(out)
}

break_multiple_variants <- function(x) {
    ALT_matrix <- str_split(x$ALT, ",", simplify = TRUE)
    AD_matrix <- str_split(x$AD, ",", simplify = TRUE)

    out <- list()

    for (i in seq_len(ncol(ALT_matrix))) {
        who <- which(ALT_matrix[, i] != "")


        out[[i]] <- data.table(
            CHROM = x[who, ]$CHROM,
            POS = x[who, ]$POS,
            ID = x[who, ]$ID,
            REF = x[who, ]$REF,
            ALT = ALT_matrix[who, i],
            DP = x[who, ]$DP,
            AD_ref = AD_matrix[who, 1],
            AD_alt = AD_matrix[who, i + 1],
            Gene_Name = x[who, ]$Gene_Name,
            Nt_alt = x[who, ]$Nt_alt,
            AA_alt = x[who, ]$AA_alt
        )
    }

    out <- rbindlist(out)

    out$POS <- as.numeric(out$POS)

    out <- out[order(out$POS), ]

    return(out)
}

compute_AF <- function(x) {
    x$AD_ref <- as.numeric(x$AD_ref)
    x$AD_alt <- as.numeric(x$AD_alt)

    x$AF <- x$AD_alt / x$DP

    x$ID <- paste(x$CHROM, x$POS, x$REF, x$ALT, sep = ";")

    return(x)
}

change_AA_abbreviations <- function(x) {
    AA_abbreviations <- data.table(
        Three_Letter = c(
            "Ala",
            "Arg",
            "Asn",
            "Asp",
            "Cys",
            "Glu",
            "Gln",
            "Gly",
            "His",
            "Ile",
            "Leu",
            "Lys",
            "Met",
            "Phe",
            "Pro",
            "Ser",
            "Thr",
            "Trp",
            "Tyr",
            "Val"
        ),
        One_Letter = c(
            "A",
            "R",
            "N",
            "D",
            "C",
            "E",
            "Q",
            "G",
            "H",
            "I",
            "L",
            "K",
            "M",
            "F",
            "P",
            "S",
            "T",
            "W",
            "Y",
            "V"
        )
    )


    x$Nt_alt <- str_remove_all(x$Nt_alt, "c\\.")
    x$AA_alt <- str_remove_all(x$AA_alt, "p\\.")

    for (i in seq_len(nrow(AA_abbreviations))) {
        x$AA_alt <- str_replace_all(
            x$AA_alt,
            AA_abbreviations[i, ]$Three_Letter,
            AA_abbreviations[i, ]$One_Letter
        )
    }

    return(x)
}

correct_Orf1ab_gene <- function(x, genes) {
    x$codon_num <- 0

    x <- x[order(x$POS), ]

    for (i in seq_len(nrow(genes))) {
        
        who <- which(
            x$POS >= genes[i, ]$start_pos & x$POS <= genes[i, ]$end_pos
        )

        x[who, ]$codon_num <- x[who, ]$POS - genes[i, ]$start_pos + 1
        x[who, ]$Gene_Name <- genes[i, ]$gene_name
        x[who, ]$codon_num <- (x[who, ]$codon_num %/% 3) +
            (x[who, ]$codon_num %% 3 > 0)
    }

    x <- x[which(x$codon_num != 0), ]

    return(x)
}

read_gene_coordinates <- function(gff_path) {
    gene_annot <- fread(gff_path, header = FALSE, sep = "\t")

    gene_annot <- gene_annot[, c(4, 5, 9), with = FALSE]

    colnames(gene_annot) <- c("start_pos", "end_pos", "gene_name")

    gene_annot[which(
        gene_annot$gene_name %in% paste0("NSP", seq_len(10))
    ), ]$gene_name <- "ORF1a"

    gene_annot[which(
        str_detect(gene_annot$gene_name, "NSP")
    ), ]$gene_name <- "ORF1b"

    gene_annot <- gene_annot[, .(
        start_pos = min(start_pos),
        end_pos = max(end_pos)
    ), by = gene_name]

    gene_annot <- gene_annot[order(gene_annot$start_pos), ]

    return(gene_annot)
}

test_that('Merge vcf returns a table',{
    expect_type(merge_vcf(
        vcf_folder = system.file("extdata","vcf-files",package = "lineagespot"
        ),
        gff3_path = system.file("extdata","NC_045512.2_annot.gff3",
                                package = "lineagespot"
        )
    ),'list')
  
    expect_type(merge_vcf(
        vcf_fls = system.file("extdata","vcf-files",
                                "SampleA_freebayes_ann.vcf",
                                package = "lineagespot"
        ),
        gff3_path = system.file("extdata","NC_045512.2_annot.gff3",
                                package = "lineagespot"
        )
    ),'list')
})

#'
#' lineagespot
#'
#' @description
#' Identify SARS-CoV-2 related mutations based on a single (or a list) of
#' variant(s) file(s)
#'
#' @param vcf_fls
#' A character vector of paths to VCF files
#'
#' @param vcf_folder
#' A path to a folder containing all VCF files
#' that will be integrated into a single table
#'
#' @param gff3_path
#' Path to GFF3 file containing SARS-CoV-2 gene coordinates.
#'
#' @param ref_folder
#' A path to a folder containing lineage reports
#'
#' @param voc
#' A character vector containing the names of the lineages of interest
#'
#' @param AF_threshold
#' A parameter indicating the AF threshold for identifying variants per sample
#'
#' @import data.table
#' @importFrom stringr str_detect str_split str_squish str_remove_all
#' @importFrom stringr str_replace_all str_detect str_to_upper str_remove
#'             str_length
#' @importFrom httr GET content
#' @importFrom VariantAnnotation readVcf info fixed
#' @importFrom SummarizedExperiment assays
#' @importFrom MatrixGenerics rowRanges
#' @importFrom utils globalVariables
#'
#' @return
#' A list of three elements;
#' * Variants' table; A data table containing all variants that are
#' included in the input VCF files
#'
#' * Lineage hits; A data table containing identified hits between the input
#' variants and outbreak.info's lineage reports
#'
#' * Lineage report; A data table with computed metrics about
#' the prevalence of the lineage of interest per sample.
#'
#'
#'
#' @export
#'
#' @examples
#'
#' results <- lineagespot(
#'     vcf_folder = system.file("extdata", "vcf-files",
#'         package = "lineagespot"
#'     ),
#'     gff3_path = system.file("extdata",
#'         "NC_045512.2_annot.gff3",
#'         package = "lineagespot"
#'     ),
#'     ref_folder = system.file("extdata", "ref",
#'         package = "lineagespot"
#'     )
#' )
#'
#' head(results$lineage.report)
#' 

lineagespot <- function(vcf_fls = NULL,
                        vcf_folder = NULL,
                        gff3_path = NULL,
                        ref_folder = NULL,
                        voc = c("B.1.617.2", "B.1.1.7", "B.1.351", "P.1"),
                        AF_threshold = 0.8
) {
    
    
    vcf_table <- merge_vcf(
        vcf_fls = vcf_fls,
        vcf_folder = vcf_folder,
        gff3_path = gff3_path
    )
    
    
    
    
    hits_table <- lineagespot_hits(
        vcf_table = vcf_table,
        ref_folder = ref_folder,
        voc = voc
    )
    
    
    
    
    lineage_report <- uniq_variants(
        hits_table = hits_table,
        AF_threshold = AF_threshold
    )
    
    
    out <- list(
        "variants.table" = vcf_table,
        "lineage.hits" = hits_table,
        "lineage.report" = lineage_report
    )
    
    
    return(out)
    
}

# Removes R CMD check NOTE regarding global variables
globalVariables(
    c(
        "DP", ".", "AD_alt", "POS", "Gene_Name", "AA_alt",
        "start_pos", "end_pos", "gene_name", "lineage",
        "AF", ""
    )
)

test_that('Test lineagespot runs properly',{
    expect_vector(lineagespot(
        vcf_folder = system.file("extdata", "vcf-files",
                                package = "lineagespot"
        ),
        gff3_path = system.file("extdata",
                                "NC_045512.2_annot.gff3",
                                package = "lineagespot"
        ),
        ref_folder = system.file("extdata", "ref",
                                package = "lineagespot"
        )
    ))
  
    expect_error(lineagespot(
        vcf_fls = system.file("extdata","vcf-files",
                                package = "lineagespot"),
        gff3_path = system.file("extdata",
                                "NC_045512.2_annot.gff3",
                                package = "lineagespot"
        ),
        ref_folder = system.file("extdata", "ref",
                                package = "lineagespot"
        )))
  
  
})

## ----vignetteSetup, echo=FALSE, message=FALSE, warning = FALSE----------------
## For links
library("BiocStyle")
## Bib setup
library("RefManageR")
## Write bibliography information
bib <- c(
    R = citation(),
    BiocStyle = citation("BiocStyle")[1],
    data.table = citation("data.table")[1],
    stringr = citation("stringr")[1]
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
    comment = "#>",
    error = FALSE,
    warning = FALSE,
    message = FALSE,
    crop = NULL
)

## ----eval = FALSE-------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#     install.packages("BiocManager")
# }
# 
# BiocManager::install("lineagespot")
# 
# ## Check that you have a valid Bioconductor installation
# BiocManager::valid()

## ----setup--------------------------------------------------------------------
library(lineagespot)

## ----eval=TRUE----------------------------------------------------------------
results <- lineagespot(vcf_folder = system.file("extdata", "vcf-files", 
                                                package = "lineagespot"),

                      gff3_path = system.file("extdata", 
                                              "NC_045512.2_annot.gff3", 
                                              package = "lineagespot"),

                      ref_folder = system.file("extdata", "ref", 
                                               package = "lineagespot"))

## -----------------------------------------------------------------------------
# overall table
head(results$variants.table)

## -----------------------------------------------------------------------------
# lineages' hits
head(results$lineage.hits)

## -----------------------------------------------------------------------------
# lineagespot report
head(results$lineage.report)

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

#'
#' get_lineage_report
#'
#' @description
#' Retrieve information about lineages' variants vie outbreak.info's API
#'
#' @param lineages
#' a character vector containing the names of the lineages of interest
#'
#' @param base.url
#' The base API URL used to search for lineage reports
#' Default value is "https://api.outbreak.info/genomics/
#' lineage-mutations?pangolin_lineage="
#'
#'
#' @return A list of data table elements of lineage reports
#' @export
#'
#' @examples
#' 
#' 
#' get_lineage_report(lineages = c("B.1.1.7", "B.1.617.2"))
#' 
#' 
get_lineage_report <- function(
    lineages, base.url =
        "https://api.outbreak.info/genomics/lineage-mutations?pangolin_lineage="
) { 
    
    lineages <- str_to_upper(lineages)

    out <- list()

    for (l in lineages) {
        strain <- GET(paste0(base.url, l))
        
        if (is.null(content(strain))){
            message(c(
                "API currently unavailable, ",
                "please use tab delimeted files instead.\n",
                "See ",
                system.file("extdata", "ref", package = "lineagespot")
            ))
            
            return(FALSE)
          
            
        }
        strain <- content(strain)

        if (strain$success) {
            strain <- strain$results[[l]]
            strain <- rbindlist(strain)

            strain$`amino acid` <- str_split(
                strain$mutation, "\\:",
                simplify = TRUE
            )[, 2]

            who <- which(strain$type != "deletion")

            strain[who, ]$`amino acid` <- str_to_upper(
                strain[who, ]$`amino acid`
            )

            strain$barcode <- paste0(strain$gene, ":", strain$`amino acid`)

            strain[who, ]$type <- "snp"

            out[[l]] <- strain
        } else {
            message(c("No data has been retrieved for ", l))
            return(FALSE)
        }
    }

    
    return(out)
}

test_that('get lineage report runs properly',{
    result <- get_lineage_report(lineages = c("B.1.1.7", "B.1.617.2"))
                                 
    if (result == FALSE){
        expect_false(result)
    }
    else{
        expect_vector(result)
    }
    
})

#' list_vcf
#'
#' @param vcf_fls
#' A character vector of paths to VCF files
#'
#' @param vcf_folder
#' A path to a folder containing all VCF files
#' that will be integrated into a single table
#'
#' @param gff3_path
#' Path to GFF3 file containing SARS-CoV-2 gene coordinates.
#'
#' @description
#' Identify VCF files from a group of files.
#'
#' @return
#' * VCF list; A list where only VCF files are stored.
#'
#'
#' @export
#'
#' @examples
#'
#' list_vcf_info <- list_vcf(
#'   vcf_folder = system.file("extdata", "vcf-files",
#'     package = "lineagespot"
#'   ),
#'   gff3_path = system.file("extdata",
#'     "NC_045512.2_annot.gff3",
#'     package = "lineagespot"
#'   )
#' )
#' print(list_vcf_info)
list_vcf <- function(vcf_fls = NULL,
                     vcf_folder = NULL,
                     gff3_path = NULL) {
  if (is.null(vcf_fls) & is.null(vcf_folder)) {
    stop("No VCF was given")
  } else if (!is.null(vcf_fls) & !is.null(vcf_folder)) {
    stop(c(
      "Please give only one source for VCF. ",
      "Other a vector of paths to VCF files",
      "or a folder containing VCF files"
    ))
  } else {
    if (is.null(vcf_fls)) {
      vcf_fls <- list.files(
        vcf_folder,
        pattern = "vcf",
        full.names = TRUE
      )

      return(vcf_fls)
    } else {
      vcf_fls <- vcf_fls[which(str_detect(vcf_fls, "vcf"))]

      if (length(vcf_fls) == 0) {
        stop("No VCF files have been provided.")
      }


      return(vcf_fls)
    }
  }
}


#' is_gff3
#'
#' @description
#' Identify whether a file is in GFF3 format.
#'
#' @param file
#' Path to GFF3 file.
#'
#' @return
#' result; TRUE if the input file is in GFF3 format, FALSE if not.
#'
#' @export
#'
#' @examples
#'
#' gff3_path <- system.file("extdata", "NC_045512.2_annot.gff3",
#'   package = "lineagespot"
#' )
#' is_gff3(gff3_path)
is_gff3 <- function(file) {
  if (is.null(file)) {
    stop(c(
      "Please provide a valid GFF3 file",
      " containing SARS-CoV-2 genes' coordinates.\n",
      "\tSee ", system.file("extdata",
        "NC_045512.2_annot.gff3",
        package = "lineagespot"
      )
    ))
  }

  if (length(file) > 1) {
    stop("Please give path of only one GFF3 file.")
  }

  res <- str_detect(file, ".gff3$")

  return(res)
}


#' list_input
#'
#' @description
#' Check the validity of input parameters from lineagespot function.
#'
#' @param vcf_fls
#' A character vector of paths to VCF files.
#'
#' @param vcf_folder
#' A path to a folder containing all VCF files
#' that will be integrated into a single table.
#'
#' @param gff3_path
#' Path to GFF3 file containing SARS-CoV-2 gene coordinates.
#'
#' @return
#' Return a character vector of paths to VCF files.
#'
#' @export
#'
#' @examples
#'
#'
#' vcflist <- list_input(
#'   vcf_folder = system.file("extdata", "vcf-files",
#'     package = "lineagespot"
#'   ),
#'   gff3_path = system.file("extdata",
#'     "NC_045512.2_annot.gff3",
#'     package = "lineagespot"
#'   )
#' )
list_input <- function(vcf_fls = NULL,
                       vcf_folder = NULL,
                       gff3_path = NULL) {
  vcf_fls <- list_vcf(vcf_fls, vcf_folder, gff3_path)


  if (length(vcf_fls) == 0) {
    stop("No VCF is found. Please insert valid input files.")
  }

  if (is_gff3(gff3_path) == FALSE) {
    stop("No valid gff3 was given. Please insert path of valid gff3.")
  }


  return(vcf_fls)
}

test_that('Input file is vcf',{
    expect_vector(list_vcf(
        vcf_folder = system.file("extdata", "vcf-files",package = "lineagespot"
    ),
        gff3_path = system.file("extdata","NC_045512.2_annot.gff3",
                                package = "lineagespot"
    )
    ))
  
    expect_equal(length(list_vcf(
        vcf_folder = system.file("extdata", "vcf-files",package = "lineagespot"
        ),
        gff3_path = system.file("extdata","NC_045512.2_annot.gff3",
                              package = "lineagespot"
        )
        )),3)
    
    expect_equal(length(list_vcf(
        vcf_folder = system.file("scripts", "vcf-files",package = "lineagespot"
        ),
        gff3_path = system.file("extdata","NC_045512.2_annot.gff3",
                              package = "lineagespot"
        )
        )),0)
    
    
    
})

test_that('Input file is gff3',{
    expect_true(is_gff3(system.file("extdata","NC_045512.2_annot.gff3",
                                    package = "lineagespot")))
    expect_false(is_gff3(system.file("extdata","vcf-files",
                                    "SampleA_freebayes_ann.vcf",
                                    package = "lineagespot")))
    expect_false(is_gff3(("/path/to/fake/gff3/file.csv")))
})

test_that('Input check runs properly',{
    expect_error( list_input(vcf_fls = system.file("extdata", "vcf-files",
                                                   "SampleA_freebayes_ann.vcf",
                                                   package = "lineagespot"
                            ),
                            vcf_folder = system.file("extdata", "vcf-files",
                                package = "lineagespot"
                            ),
                            gff3_path = system.file("extdata",
                                "NC_045512.2_annot.gff3",
                                package = "lineagespot"
                            )
                            ))

    expect_error( list_input(vcf_fls = NULL,

                            vcf_folder = NULL,

                            gff3_path = system.file("extdata",
                                                    "NC_045512.2_annot.gff3",
                                                    package = "lineagespot"
                            )))

    expect_error( list_input(vcf_folder = system.file("extdata", "vcf-files",
                                                     package = "lineagespot"
                            ),
                            gff3_path = system.file("extdata", "vcf-files",
                                                    "SampleA_freebayes_ann.vcf",
                                                    package = "lineagespot"
                            )))


    expect_vector( list_input(vcf_folder = system.file("extdata", "vcf-files",
                                                    package = "lineagespot"
                            ),
                            gff3_path = system.file("extdata",
                                                    "NC_045512.2_annot.gff3",
                                                    package = "lineagespot"
                            )))

})

---
title: "lineagespot User Guide"
author:
- name: Nikolaos Pechlivanis
  affiliation: Institute of Applied Biosciences, 
    Centre for Research and Technology Hellas, Thessaloniki, GR
  email: nikosp41@certh.gr
- name: Maria Tsagiopoulou
  affiliation: Institute of Applied Biosciences, 
    Centre for Research and Technology Hellas, Thessaloniki, GR
- name: Maria Christina Maniou
  affiliation: Institute of Applied Biosciences, 
    Centre for Research and Technology Hellas, Thessaloniki, GR
- name: Anastasis Togkousidis
  affiliation: Institute of Applied Biosciences, 
    Centre for Research and Technology Hellas, Thessaloniki, GR
- name: Evangelia Mouchtaropoulou
  affiliation: Institute of Applied Biosciences, 
    Centre for Research and Technology Hellas, Thessaloniki, GR
- name: Taxiarchis Chassalevris
  affiliation: School of Veterinary Medicine, 
    Aristotle University of Thessaloniki, GR
- name: Serafeim Chaintoutis
  affiliation: School of Veterinary Medicine, 
    Aristotle University of Thessaloniki, GR
- name: Chrysostomos Dovas
  affiliation: School of Veterinary Medicine, 
    Aristotle University of Thessaloniki, GR
- name: Maria Petala
  affiliation: Dept. of Civil Engineering, 
    Aristotle University of Thessaloniki, GR
- name: Margaritis Kostoglou
  affiliation: Dept. of Chemistry, 
    Aristotle University of Thessaloniki, GR
- name: Thodoris Karapantsios
  affiliation: Dept. of Chemistry, 
    Aristotle University of Thessaloniki, GR
- name: Stamatia Laidou
  affiliation: Institute of Applied Biosciences, 
    Centre for Research and Technology Hellas, Thessaloniki, GR
- name: Elisavet Vlachonikola
  affiliation: Institute of Applied Biosciences, 
    Centre for Research and Technology Hellas, Thessaloniki, GR
- name: Anastasia Chatzidimitriou
  affiliation: Institute of Applied Biosciences, 
    Centre for Research and Technology Hellas, Thessaloniki, GR
- name: Agis Papadopoulos
  affiliation: EYATH S.A., 
    Thessaloniki Water Supply and Sewerage Company S.A., Thessaloniki, GR
- name: Nikolaos Papaioannou
  affiliation: School of Veterinary Medicine, 
    Aristotle University of Thessaloniki, GR
- name: Anagnostis Argiriou
  affiliation: Institute of Applied Biosciences, 
    Centre for Research and Technology Hellas, Thessaloniki, GR
- name: Fotis E. Psomopoulos
  affiliation: Institute of Applied Biosciences, 
    Centre for Research and Technology Hellas, Thessaloniki, GR

date: "`r BiocStyle::doc_date()`"
package: "`r BiocStyle::pkg_ver('lineagespot')`"
output: 
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{lineagespot User Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r vignetteSetup, echo=FALSE, message=FALSE, warning = FALSE}
## For links
library("BiocStyle")
## Bib setup
library("RefManageR")
## Write bibliography information
bib <- c(
    R = citation(),
    BiocStyle = citation("BiocStyle")[1],
    data.table = citation("data.table")[1],
    stringr = citation("stringr")[1]
)
```


<style>
<!-- h1, h2, h3, h4 { -->
<!--   color:#17247a; -->
<!-- } -->

strong {
    color:#eb6b1c;
}

</style>

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
    comment = "#>",
    error = FALSE,
    warning = FALSE,
    message = FALSE,
    crop = NULL
)
```
# Introduction

`lineagespot` is a framework written in R, and aims to identify 
SARS-CoV-2 related mutations based on a single (or a list) of variant(s) 
file(s) (i.e., variant calling format). The method can facilitate the 
detection of SARS-CoV-2 lineages in wastewater samples using next 
generation sequencing, and attempts to infer the potential distribution 
of the SARS-CoV-2 lineages. 

# Quick start

## Installation

`lineagespot` is distributed as a [Bioconductor](https://www.bioconductor.org/) 
package and requires `R` (version "4.1"), which can be installed on any 
operating system from [CRAN](https://cran.r-project.org/), and 
Bioconductor (version "3.14").

To install `lineagespot` package enter the following commands in 
your `R` session:

```{r eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("lineagespot")

## Check that you have a valid Bioconductor installation
BiocManager::valid()
```

## Raw data analysis

Example fastq files are provided through 
[zenodo](https://zenodo.org/badge/DOI/10.5281/zenodo.4564182.svg). 
For the pre processing steps of them, the bioinformatics analysis pipeline is 
provided [here](../inst/scripts/raw-data-analysis.md).

## Running lineagespot 

Once `lineagespot` is successfully installed, it can be loaded as follow:

```{r setup}
library(lineagespot)
```

`lineagespot` can be run by calling one function that implements the overall 
pipeline:


```{r eval=TRUE}
results <- lineagespot(vcf_folder = system.file("extdata", "vcf-files", 
                                                package = "lineagespot"),

                      gff3_path = system.file("extdata", 
                                              "NC_045512.2_annot.gff3", 
                                              package = "lineagespot"),

                      ref_folder = system.file("extdata", "ref", 
                                               package = "lineagespot"))
```

## Explore the results

The function returns three tables:

- An overall variant table containing all variants included in the input VCF 
files, along with the related information (gene, location etc.)  

```{r}
# overall table
head(results$variants.table)
```
- A table with the identified overlaps/hits between the variant table and the 
given lineage reports.

```{r}
# lineages' hits
head(results$lineage.hits)
```

- A lineage report table where metrics for the abundance of each lineage are 
computed. 
  To this end, the mean AF (allele frequence) of the variants per lineage, 
  the mean AF of the unique variants per lineage and the non-zero min. AF of the
  lineage's unique variants are computed.
  Moreover given a AF threshold the number of variants in each sample is 
  computed along with the resulting proportion (Number of variants to the number
  of lineage rules).

```{r}
# lineagespot report
head(results$lineage.report)
```

# Session info {.unnumbered}

Here is the output of `sessionInfo()` on the system on which this document was
compiled running pandoc ``r rmarkdown::pandoc_version()``:

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

# Detecting SARS-CoV-2 lineages and mutational load in municipal wastewater; raw data analysis


## Steps


### Quality and adapter trimming

```
mkdir trimmedData/

trim_galore --path_to_cutadapt /path/to/cutadapt \
    -o trimmedData/ \
    --paired SampleA_R1_001.fastq.gz SampleA_R2_001.fastq.gz > SampleA_trimming.output 2>&1
```

### Mapping to reference genome

```
minimap2 -t 8 \
    -ax sr NC_045512.fasta \
    trimmedData/SampleA_R1_001_val_1.fq.gz \
    trimmedData/SampleA_R2_001_val_2.fq.gz > SampleA.sam
```

### BAM file management

```
samtools view -@ 8 -Sb SampleA.sam > SampleA.bam
samtools sort -@ 8 -n SampleA.bam -o SampleA_sorted.bam

samtools view -@ 8 -u -f 1 -F 12 SampleA_sorted.bam > SampleA_mapped.bam
samtools sort -@ 8 SampleA_mapped.bam -o SampleA_sorted_mapped.bam
```

### Primer trimming

```
samtools index SampleA_sorted_mapped.bam

ivar trim \
    -b nCoV-2019.primer.bed \
    -q 15 \
    -m 200 \
    -s 4 \
    -i SampleA_sorted_mapped.bam \
    -p SampleA_sorted_mapped_trimmed.bam > SampleA_ivar.out
```

### Re-mapping to reference genome

```
samtools sort -@ 8 -n SampleA_sorted_mapped_trimmed.bam -o SampleA_sorted_mapped_trimmed_f.bam
bedtools bamtofastq -i SampleA_sorted_mapped_trimmed_f.bam -fq SampleA_R1_mapped.fastq -fq2 SampleA_R2_mapped.fastq

minimap2 -t 8 -ax sr NC_045512.fasta SampleA_R1_mapped.fastq SampleA_R2_mapped.fastq > SampleA_f.sam

samtools view -@ 8 -Sb SampleA_f.sam > SampleA_f.bam 
samtools sort -@ 8 -o SampleA_sorted_f.bam SampleA_f.bam 
```

### Filtering out duplicates

```
java -jar picard.jar MarkDuplicates I=SampleA_sorted_f.bam O=SampleA_sorted_uniq.bam M=SampleA_dup_metrics.txt
```

### Produce and annotate VCF file

```
freebayes \
    -f NC_045512.fasta \
    -F 0.01 \
    -C 1 \
    --pooled-continuous SampleA_sorted_uniq.bam > SampleA_freebayes.vcf

snpEff ann NC_045512.2 SampleA_freebayes.vcf > SampleA_freebayes_ann.vcf
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uniq_variants.R
\name{uniq_variants}
\alias{uniq_variants}
\title{uniq_variants}
\usage{
uniq_variants(hits_table = NULL, AF_threshold = 0.8)
}
\arguments{
\item{hits_table}{A tab-delimited table containing the identified overlaps/hits between the
input files and the lineages' reports. This input is generated by the
\code{lineagespot_hits} function.}

\item{AF_threshold}{A parameter indicating the AF threshold that is going to applied in order to
identify the presence or not of a variant. This is used to compute the number
of variants in a sample and eventually the proportion of a lineage.}
}
\value{
A data table with metrics assessing the
abundance of every lineage in each samples
}
\description{
Lineage report for variants overlapping
}
\examples{

variants_table <- merge_vcf(
    vcf_folder = system.file("extdata", "vcf-files",
        package = "lineagespot"
    ),
    gff3_path = system.file("extdata",
        "NC_045512.2_annot.gff3",
        package = "lineagespot"
    )
)

lineage_hits_table <- lineagespot_hits(
    vcf_table = variants_table,
    ref_folder = system.file("extdata", "ref",
        package = "lineagespot")
)

report <- uniq_variants(hits_table = lineage_hits_table)
head(report)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_lineage_report.R
\name{get_lineage_report}
\alias{get_lineage_report}
\title{get_lineage_report}
\usage{
get_lineage_report(
  lineages,
  base.url = "https://api.outbreak.info/genomics/lineage-mutations?pangolin_lineage="
)
}
\arguments{
\item{lineages}{a character vector containing the names of the lineages of interest}

\item{base.url}{The base API URL used to search for lineage reports
Default value is "https://api.outbreak.info/genomics/
lineage-mutations?pangolin_lineage="}
}
\value{
A list of data table elements of lineage reports
}
\description{
Retrieve information about lineages' variants vie outbreak.info's API
}
\examples{


get_lineage_report(lineages = c("B.1.1.7", "B.1.617.2"))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lineagespot.R
\name{lineagespot}
\alias{lineagespot}
\title{lineagespot}
\usage{
lineagespot(
  vcf_fls = NULL,
  vcf_folder = NULL,
  gff3_path = NULL,
  ref_folder = NULL,
  voc = c("B.1.617.2", "B.1.1.7", "B.1.351", "P.1"),
  AF_threshold = 0.8
)
}
\arguments{
\item{vcf_fls}{A character vector of paths to VCF files}

\item{vcf_folder}{A path to a folder containing all VCF files
that will be integrated into a single table}

\item{gff3_path}{Path to GFF3 file containing SARS-CoV-2 gene coordinates.}

\item{ref_folder}{A path to a folder containing lineage reports}

\item{voc}{A character vector containing the names of the lineages of interest}

\item{AF_threshold}{A parameter indicating the AF threshold for identifying variants per sample}
}
\value{
A list of three elements;
\itemize{
\item Variants' table; A data table containing all variants that are
included in the input VCF files
\item Lineage hits; A data table containing identified hits between the input
variants and outbreak.info's lineage reports
\item Lineage report; A data table with computed metrics about
the prevalence of the lineage of interest per sample.
}
}
\description{
Identify SARS-CoV-2 related mutations based on a single (or a list) of
variant(s) file(s)
}
\examples{

results <- lineagespot(
    vcf_folder = system.file("extdata", "vcf-files",
        package = "lineagespot"
    ),
    gff3_path = system.file("extdata",
        "NC_045512.2_annot.gff3",
        package = "lineagespot"
    ),
    ref_folder = system.file("extdata", "ref",
        package = "lineagespot"
    )
)

head(results$lineage.report)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lineagespot_hits.R
\name{lineagespot_hits}
\alias{lineagespot_hits}
\title{lineagespot_hits}
\usage{
lineagespot_hits(
  vcf_table = NULL,
  ref_folder = NULL,
  voc = c("B.1.617.2", "B.1.1.7", "B.1.351", "P.1")
)
}
\arguments{
\item{vcf_table}{A tab-delimited table containing all variants for all samples. This input
is generated by the \code{merge_vcf} function.}

\item{ref_folder}{A path to lineages' reports}

\item{voc}{A character vector containing the names of the lineages of interest}
}
\value{
A data table containing all identified SARS-CoV-2 variants
based on the provided reference files
}
\description{
Find overlapping variants with SARS-CoV-2 reference lineages
coming from outbreak.info reports
}
\examples{

variants_table <- merge_vcf(
    vcf_folder = system.file("extdata",
        "vcf-files",
        package = "lineagespot"
    ),
    gff3_path = system.file("extdata",
        "NC_045512.2_annot.gff3",
        package = "lineagespot"
    )
)

# retrieve lineage reports using outbreak.info's API

# use user-specified references
lineage_hits_table <- lineagespot_hits(
    vcf_table = variants_table,
    ref_folder = system.file("extdata", "ref",
        package = "lineagespot"
    )
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_vcf.R
\name{merge_vcf}
\alias{merge_vcf}
\title{merge_vcf}
\usage{
merge_vcf(vcf_fls = NULL, vcf_folder = NULL, gff3_path = NULL)
}
\arguments{
\item{vcf_fls}{A list of paths to VCF files}

\item{vcf_folder}{A path to a folder containing all VCF file that
will be integrated into a single table}

\item{gff3_path}{Path to GFF3 file}
}
\value{
A data table contaiing all variants from each sample of the input VCF files
}
\description{
Merge Variant Calling Format (VCF) files into a single tab-delimited table
}
\examples{

merge_vcf(
    vcf_folder = system.file("extdata",
        "vcf-files",
        package = "lineagespot"
    ),
    gff3_path = system.file("extdata",
        "NC_045512.2_annot.gff3",
        package = "lineagespot"
    )
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input_checking.R
\name{list_vcf}
\alias{list_vcf}
\title{list_vcf}
\usage{
list_vcf(vcf_fls = NULL, vcf_folder = NULL, gff3_path = NULL)
}
\arguments{
\item{vcf_fls}{A character vector of paths to VCF files}

\item{vcf_folder}{A path to a folder containing all VCF files
that will be integrated into a single table}

\item{gff3_path}{Path to GFF3 file containing SARS-CoV-2 gene coordinates.}
}
\value{
\itemize{
\item VCF list; A list where only VCF files are stored.
}
}
\description{
Identify VCF files from a group of files.
}
\examples{

list_vcf_info <- list_vcf(
  vcf_folder = system.file("extdata", "vcf-files",
    package = "lineagespot"
  ),
  gff3_path = system.file("extdata",
    "NC_045512.2_annot.gff3",
    package = "lineagespot"
  )
)
print(list_vcf_info)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input_checking.R
\name{is_gff3}
\alias{is_gff3}
\title{is_gff3}
\usage{
is_gff3(file)
}
\arguments{
\item{file}{Path to GFF3 file.}
}
\value{
result; TRUE if the input file is in GFF3 format, FALSE if not.
}
\description{
Identify whether a file is in GFF3 format.
}
\examples{

gff3_path <- system.file("extdata", "NC_045512.2_annot.gff3",
  package = "lineagespot"
)
is_gff3(gff3_path)
}
